#ifndef _GENERAL_H
#define _GENERAL_H

//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include <string.h>
#include <stdlib.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <stddef.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/times.h>
#include <sys/unistd.h>
#include "stm32_ub_ws2812.h"

//--------------------------------------------------------------
// Defines
//--------------------------------------------------------------

#define DEBUG       	1           ///< Debug Mode Flagge
#define TERMINAL	1

#define VER_BUILD	1317		///< Version Build Number
#define VER_MINOR	0 			///< Version Minor
#define VER_MAJOR	1 			///< Version Major

#define TWI_ML_ADDRESS  (0x38<<1) ///< I2C Addresse (shifted!!!) [0x38]
#define TWI_SCAN_INTERVAL 60   	///< Interval um nach neuen I²C Werten zu scannen (in ms)
#define TWI_ID_HIGH 0x44		///< I²C Identifikations ID High Byte zur Erkennung des Bedienteils
#define TWI_ID_LOW 0x94			///< I²C Identifikations ID Low Byte zur Erkennung des Bedienteils

#define MAX_STRLEN 12 // this is the maximum string length of our string in characters
#define MAX_EFFECTS 16      ///< Anzahl an LED Effekten
#define MAX_HUE 359
#define MAX_SAT 255
#define MAX_DELAY 100
#define MAX_BRIGHT 255
#define LED_FADEOUT_SPEED 20
#define MAXCOMMANDSCHARS 80

#define MAX_CHANNELS 1

// Effect 1
#define NUMFADE     5           ///< Mindest Wert für Fadeout
#define BRIGHT      60          ///< Wert für Zufällige Helligkeit
#define MINBRIGHT   32          ///< Minimale Helligkeit für RGB
#define OUTDELAY    20          ///< Delay in ms
#define NUMFLOW     2          ///< Anzahl an "flows" für Effekt Welle


//--------------------------------------------------------------
// Structures
//--------------------------------------------------------------

typedef struct {
  int NewCommand;
  unsigned char Command[MAXCOMMANDSCHARS];
  int CommandCount;
} UART;

typedef struct CHANNEL {
    union {
        struct {
            union {
                uint8_t thisprogram;
                uint8_t program;
                uint8_t p;
            };
            union {
                uint8_t thissat;
                uint16_t saturation;
                uint8_t s;
            };
            union {
                uint8_t thisvalue;
                uint8_t brightness;
                uint8_t b;
            };
            union {
                uint16_t thishue;
                uint16_t hue;
                uint16_t h;
            };
            union {
                uint8_t thisdelay;
                uint8_t delay;
                uint8_t d;
            };

            uint8_t ibright;
            uint8_t bouncedirection;
            uint16_t BOTTOM_INDEX;
            uint16_t MIDDLE_INDEX;
            uint16_t EVENODD;
            uint16_t idex;
            uint16_t ihue;
            uint8_t isat;

            bool bgColorSet;
            char bgColor[3];

            // Effect Welle 1 Variablen 
            uint16_t I, J, PixelP[NUMFLOW];
            uint8_t PixelR[NUMFLOW], PixelG[NUMFLOW], PixelB[NUMFLOW], C;

            // Effect 2
            uint8_t e2_wave;
            uint8_t e2_index;
            uint8_t fade;

            // Musicprogramm 1 Variablen
            long max_val0, max_val1, max_val2;
            uint8_t blank;
            int j;
            uint16_t offset;
            int welle;
            uint16_t wellen;

            uint8_t delay_copy;

            uint32_t _thisdelay;

            volatile long led_count;
        };
    };
} CHANNEL;

enum CONTROL_TYPE {
  BLUETOOTH_CONNECTION,
  TWI_CONNECTION
};

extern uint8_t control_type;

extern WS2812_RGB_t *led[4];

#endif // _GENERAL_H
