/* Includes ------------------------------------------------------------------*/
#include "main.h"

#include "stm32f10x_gpio.h"
#include "stm32f10x_rcc.h"
#include "stm32f10x_tim.h"
#include "stm32f10x_dma.h"

#include "stm32_ub_led.h"
#include "stm32_ub_ws2812.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
WS2812_RGB_t *led[4];

ErrorStatus HSEStartUpStatus;
RCC_ClocksTypeDef RCC_ClockFreq;
uint8_t control_type;

/* Private function prototypes -----------------------------------------------*/
void RCC_Configuration(void);
void NVIC_Configuration(void);
void Delay(vu32 nCount);

/* Private functions ---------------------------------------------------------*/

/*******************************************************************************
* Function Name  : main
* Description    : Main program.
* Input          : None
* Output         : None
* Return         : None
* 
* 	// TIM3-CH1 = PA6 / PC6 / PB4 - ok
	// TIM3-CH2 = PA7 / PC7 / PB5 - ok
	// TIM3-CH3 = PB0 / PC8 - ok
	// TIM3-CH4 = PB1 / PC9 - 
*******************************************************************************/
int main(void)
{
	/* Configure the system clocks */
	RCC_Configuration();
    
	/* NVIC Configuration */
	NVIC_Configuration();
	
	init_USART1(115200);	// USART1 initialisieren
	
	UB_Led_Init();			// Debug LEDs initialisieren
	
	UB_WS2812_Init();

	WS2812_CHAIN_LEN.ch1 = 5;
	WS2812_CHAIN_LEN.ch2 = 5;
	WS2812_CHAIN_LEN.ch3 = 5;
	WS2812_CHAIN_LEN.ch4 = 5;
	
	UB_Led_On(LED_GREEN);
	
	char buf[128];
	RCC_GetClocksFreq(&RCC_ClockFreq);
	sprintf(buf, "Running at following clocks: \r\n  SYSCLK = %ld MHz\r\n  HCLK   = %ld MHz\r\n  PCLK1  = %ld MHz\r\n  PCLK2  = %ld MHz\r\n\r\n", RCC_ClockFreq.SYSCLK_Frequency/1000000, RCC_ClockFreq.HCLK_Frequency/1000000, RCC_ClockFreq.PCLK1_Frequency/1000000, RCC_ClockFreq.PCLK2_Frequency/1000000);
	printf(buf);
	
	d_printArray();
	
	/* Infinite loop */
	while (1)
	{
		UB_WS2812_SetChannel(1);
		UB_WS2812_All_Led_RGB((WS2812_RGB_t) {128,0,0}, 1);
		
		UB_WS2812_SetChannel(2);
		UB_WS2812_All_Led_RGB((WS2812_RGB_t) {0,128,0}, 1);
		
		UB_WS2812_SetChannel(3);
		UB_WS2812_All_Led_RGB((WS2812_RGB_t) {0,0,128}, 1);
		
		UB_WS2812_SetChannel(4);
		UB_WS2812_All_Led_RGB((WS2812_RGB_t) {70,70,70}, 1);
		
		parseCommands();
		Delay(500000);
	}
}

/*******************************************************************************
* Function Name  : RCC_Configuration
* Description    : Configures the different system clocks.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void RCC_Configuration(void)
{
  // RCC system reset(for debug purpose)
  RCC_DeInit();

  // Enable HSE
  RCC_HSEConfig(RCC_HSE_ON);

  // Wait till HSE is ready
  HSEStartUpStatus = RCC_WaitForHSEStartUp();

  if(HSEStartUpStatus == SUCCESS)
  {
    // Enable Prefetch Buffer
    FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

    // Flash 2 wait state
    FLASH_SetLatency(FLASH_Latency_2);
 	
	// HCLK = SYSCLK
    RCC_HCLKConfig(RCC_SYSCLK_Div1); 
  
    // PCLK2 = HCLK
    RCC_PCLK2Config(RCC_HCLK_Div1); 

    // PCLK1 = HCLK/2
    RCC_PCLK1Config(RCC_HCLK_Div2);

    // PLLCLK = 8MHz * 9 = 72 MHz
    RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9);

    // Enable PLL
    RCC_PLLCmd(ENABLE);

    // Wait till PLL is ready
    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET)
    {
    }

    // Select PLL as system clock source
    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

    // Wait till PLL is used as system clock source
    while(RCC_GetSYSCLKSource() != 0x08)
    {
    }
  } 
  /* Output HSE clock on MCO pin ---------------------------------------------*/
  /*
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);
  GPIO_InitTypeDef  GPIO_InitStructure;
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_Init(GPIOA, &GPIO_InitStructure);
  RCC_MCOConfig(RCC_MCO_HSE);
  */
}

/*******************************************************************************
* Function Name  : NVIC_Configuration
* Description    : Configures Vector Table base location.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void NVIC_Configuration(void)
{
#ifdef  VECT_TAB_RAM  
  /* Set the Vector Table base location at 0x20000000 */ 
  NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0); 
#else  /* VECT_TAB_FLASH  */
  /* Set the Vector Table base location at 0x08000000 */ 
  NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);   
#endif
}

/*******************************************************************************
* Function Name  : Delay
* Description    : Inserts a delay time.
* Input          : nCount: specifies the delay time length.
* Output         : None
* Return         : None
*******************************************************************************/
void Delay(vu32 nCount)
{
  for(; nCount != 0; nCount--);
}

#ifdef  DEBUG
/*******************************************************************************
* Function Name  : assert_failed
* Description    : Reports the name of the source file and the source line number
*                  where the assert_param error has occurred.
* Input          : - file: pointer to the source file name
*                  - line: assert_param error line source number
* Output         : None
* Return         : None
*******************************************************************************/
void assert_failed(u8* file, u32 line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  printf("Wrong parameters value: file %s on line %d\r\n", (char*)file, (int)line);
  /* Infinite loop */
  while (1)
  {
  }
}
#endif

