//--------------------------------------------------------------
// File     : stm32_ub_led.c
// Datum    : 24.10.2013
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F429
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : GPIO
// Funktion : LED Funktionen
//--------------------------------------------------------------

//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_led.h"
#include "stm32f10x_gpio.h"
#include "stm32f10x_rcc.h"

//--------------------------------------------------------------
// Definition aller LEDs
// Reihenfolge wie bei LED_NAME_t
//
// Init : [LED_OFF,LED_ON]
//--------------------------------------------------------------
LED_t LED[] = {
  // Name    ,PORT , PIN       , CLOCK              , Init
  {LED_GREEN,GPIOA,GPIO_Pin_5,RCC_APB2Periph_GPIOA,LED_OFF}, 
  {LED_DEBUG_1,GPIOC,GPIO_Pin_8,RCC_APB2Periph_GPIOC,LED_OFF},
  {LED_DEBUG_2,GPIOC,GPIO_Pin_5,RCC_APB2Periph_GPIOC,LED_OFF} 
  //{LED_DEBUG_3,GPIOC,GPIO_Pin_15,RCC_APB2Periph_GPIOC,LED_OFF}, 
};



//--------------------------------------------------------------
// Init aller LEDs
//--------------------------------------------------------------
void UB_Led_Init(void)
{
  GPIO_InitTypeDef  GPIO_InitStructure;
  LED_NAME_t led_name;
  
  for(led_name=0;led_name<LED_ANZ;led_name++) {
    // Clock Enable
    RCC_APB2PeriphClockCmd(LED[led_name].LED_CLK, ENABLE);

    // Config als Digital-Ausgang
    GPIO_InitStructure.GPIO_Pin = LED[led_name].LED_PIN;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP; // | GPIO_Mode_IPU;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(LED[led_name].LED_PORT, &GPIO_InitStructure);

    // Default Wert einstellen
    if(LED[led_name].LED_INIT==LED_OFF) {
      UB_Led_Off(led_name);
    }
    else {
      UB_Led_On(led_name);
    }
  }
}


//--------------------------------------------------------------
// LED ausschalten
//--------------------------------------------------------------
void UB_Led_Off(LED_NAME_t led_name)
{
  GPIO_ResetBits(LED[led_name].LED_PORT, LED[led_name].LED_PIN);
}

//--------------------------------------------------------------
// LED einschalten
//--------------------------------------------------------------
void UB_Led_On(LED_NAME_t led_name)
{
  GPIO_SetBits(LED[led_name].LED_PORT, LED[led_name].LED_PIN);
} 

//--------------------------------------------------------------
// LED toggeln
//--------------------------------------------------------------
void UB_Led_Toggle(LED_NAME_t led_name)
{
	if (GPIO_ReadInputDataBit(LED[led_name].LED_PORT, LED[led_name].LED_PIN) == 0)
		GPIO_SetBits(LED[led_name].LED_PORT, LED[led_name].LED_PIN);
	else 
		GPIO_ResetBits(LED[led_name].LED_PORT, LED[led_name].LED_PIN);
}	

//--------------------------------------------------------------
// LED ein- oder ausschalten
//--------------------------------------------------------------
void UB_Led_Switch(LED_NAME_t led_name, LED_STATUS_t wert)
{
  if(wert==LED_OFF) {
    UB_Led_Off(led_name);
  }
  else {
    UB_Led_On(led_name);
  }
}
